@@ -1,6 +1,42 @@
 /* main method routines */
+
+/* See http://developer.r-project.org/blosxom.cgi/R-devel/2019/08/29#n2019-08-29
+   for what USE_FC_LEN_T is doing and for why see
+   https://developer.r-project.org/Blog/public/2019/05/15/gfortran-issues-with-lapack/index.html 
+   
+   In a nutshell, the mechanism used to call BLAS/LAPACK from C (by everyone, not just R) is not 
+   technically supported by the Fortran standard. Fortran needs to know how long strings are (they 
+   are not null terminated) so the lengths are passed as hidden extra function arguments. BLAS/LAPACK
+   only ever uses single character strings, so it never needs to access the string lengths and it is 
+   then no problem that they are missing (they are at the end of the call arguments), so they are simply 
+   not passed in the C call. This was no problme until Gfortran decided to optimize the process of calling 
+   a function with the same argument list as the calling function. Basically it passed the call stack of 
+   the calling function to the called function assuming that it contained the string lengths - as it 
+   didn't this caused stack corruption. 
+
+   The solution is to pass the unit string lengths explicitly using FCONE defined in Rconfig.h if 
+   USE_FC_LEN_T is defined. This mechanism is needed since it is compiler specific what type is used 
+   to pass the string lengths (what happens then if BLAS/LAPACK and R are compiled using different 
+   extra argument types is unclear to me, but no problems of this sort are currently known in any case 
+   to get an actual problem the LAPACK/BLAS compiler would have to be using a different number of bytes 
+   to the R compiler). 
+
+   In practice when calling BLAS/LAPACK macro FCONE has to be added to the end of the call as
+   many times as there are character arguments to the call. mat.c has many examples.
+
+*/
+
+#define USE_FC_LEN_T
 #include <Rinternals.h>
 #include <Rconfig.h>
+#include <R_ext/BLAS.h>
+#include <R_ext/Lapack.h>
+/* If we are compiling with a version of R before FCONE and the explicit supplying of extra arguments 
+   was introduced, then FCONE has to be defined */ 
+#ifndef FCONE
+#define FCONE
+#endif
+
 /* Most compilers with openMP support supply 
    a pre-defined compiler macro _OPENMP. Following 
    facilitates selective turning off (by testing value 
@@ -27,7 +63,7 @@
 /* BUT, this can mess up valgrinding for memory error checking - problems are 
    sometimes missed because standard allocation is being circumvented. Then errors can 
    corrupt R memory management without detection and trigger nothing until R
-   messes up internally becuase of corruption, which then makes it look as if
+   messes up internally because of corruption, which then makes it look as if
    R is generating the problem. Hence better to reset for checking. Also sizing
    errors in .C often generate no obvious valgrind error.*/
 //#define CALLOC calloc
@@ -92,13 +128,16 @@ void XWXd(double *XWX,double *X,double *w,int *k,int *ks, int *m,int *p, int *n,
 void XWXd0(double *XWX,double *X,double *w,int *k,int *ks, int *m,int *p, int *n, int *nx, 
           int *ts, int *dt, int *nt,double *v,int *qc,int *nthreads,int *ar_stop,
           int *ar_row,double *ar_weights);
-void XWyd(double *XWy,double *y,double *X,double *w,int *k, int *ks, int *m,int *p, int *n, 
+void XWXd1(double *XWX,double *X,double *w,int *k,int *ks, int *m,int *p, int *n, int *nx, int *ts, 
+	   int *dt, int *nt,double *v,int *qc,int *nthreads,int *ar_stop,int *ar_row,double *ar_weights,
+	   int *rs, int *cs, int *nrs, int *ncs);
+void XWyd(double *XWy,double *y,double *X,double *w,int *k, int *ks, int *m,int *p, int *n,int *cy, 
 	  int *nx, int *ts, int *dt, int *nt,double *v,int *qc,
-          int *ar_stop,int *ar_row,double *ar_weights);
+          int *ar_stop,int *ar_row,double *ar_weights,int *cs,int *ncs);
 void Xbd(double *f,double *beta,double *X,int *k, int *ks, int *m,int *p, int *n, 
-	 int *nx, int *ts, int *dt, int *nt,double *v,int *qc,int *bc);
+	 int *nx, int *ts, int *dt, int *nt,double *v,int *qc,int *bc,int *cs,int *ncs);
 void diagXVXt(double *diag,double *V,double *X,int *k,int *ks,int *m,int *p, int *n, 
-	      int *nx, int *ts, int *dt, int *nt,double *v,int *qc,int *pv,int *nthreads);
+	      int *nx, int *ts, int *dt, int *nt,double *v,int *qc,int *pv,int *cv,int *nthreads,int *cs,int *ncs,int *rs,int *nrs);
 
 /* various service routines */
 
@@ -115,7 +154,6 @@ void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol,
 void RuniqueCombs(double *X,int *ind,int *r, int *c);
 void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,double *Afd,double *Sd,int *off,int *dim,double *theta, int *m,int *nar);
 void RMonoCon(double *Ad,double *bd,double *xd,int *control,double *lower,double *upper,int *n);
-/*void MinimumSeparation(double *gx,double *gy,int *gn,double *dx,double *dy, int *dn,double *dist);*/
 void MinimumSeparation(double *x,int *n, int *d,double *t,int *m,double *dist);
 void rksos(double *x,int *n,double *eps);
 void pivoter(double *x,int *r,int *c,int *pivot, int *col, int *reverse);
